package com.gct.centralbase.config;

import com.gct.centralbase.job.CalErrorDataJob;
import org.quartz.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.io.IOException;
import java.util.Date;

/**
 * class name: QuartzTaskConfig
 *
 * @author lloyd
 * @version 1.0
 * @since 2021/5/21 上午10:21
 */
@Configuration
public class QuartzTaskConfig {

    private static final   String jobName = "calErrorDataJob";

    private static final   String jobGroup = "CALERRORDATAJOB_GROUP";

    /**
     * 基础数据异常JobDetail
     * @return
     */
    @Bean
    public JobDetail CalAoidLiqErrorJob() {
        try {
            return JobBuilder.newJob(CalErrorDataJob.class)
                    .withIdentity(jobName, jobGroup)
                    .storeDurably()
                    .build();
        }catch (Exception e)
        {
            Logger log = LoggerFactory.getLogger(this.getClass());
            log.error("Create job failed==>jobName:{},jobGroup:{}-----cause by msg:{}", jobName, jobGroup);
            throw e;
        }

    }

    /**
     * 基础数据异常
     * @return
     */
    @Bean
    public Trigger CalAoidLiqErrorJobTrigger() throws IOException {
        try {
            SimpleScheduleBuilder scheduleBuilder = SimpleScheduleBuilder.simpleSchedule()
                    .withIntervalInHours(4);
            scheduleBuilder.repeatForever();
            return TriggerBuilder.newTrigger()
                    .forJob(jobName, jobGroup)
                    .startNow()
                    //.startAt(new Date(new Date().getTime() + 5000))
                    .withSchedule(scheduleBuilder).build();
        }catch (Exception e)
        {
            Logger log=LoggerFactory.getLogger(this.getClass());
            log.error("Create trigger failed==>jobName:{},jobGroup:{}-----cause by msg:{}",jobName, jobGroup, e.getMessage());
            throw e;
        }
    }
}
