package com.gct.centralbase.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.gct.centralbase.entity.CbPcOrganization;
import com.gct.centralbase.service.CbPcOrganizationService;
import com.gct.common.core.result.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 组织机构 前端控制器
 * </p>
 *
 * @author wsy
 * @since 2021-01-19
 */
@Api(value = "CbPcOrganizationController",description = "基础数据前端-组织架构导航")
@RestController
@RequestMapping("/org")
public class CbPcOrganizationController {

    @Autowired
    private CbPcOrganizationService cbPcOrganizationService;

    //only feign
    @GetMapping("/getParAndSonOrg")
    public String getParAndSonOrg(@RequestParam JSONArray orgStr){
        return cbPcOrganizationService.getParAndSonOrg(orgStr);
    }

    //only feign
    @GetMapping("/getParentOrgAsc")
    public List<CbPcOrganization> getParentOrgAsc(@RequestParam String wellId){
        return cbPcOrganizationService.getParentOrgAsc(wellId);
    }

    @ApiOperation(value = "getOrganization(获取组织架构数据)", notes = "获取组织架构数据")
    @GetMapping("/org")
    public Result getOrganization(@RequestParam(required = false,value = "parent_id") String orgStr){
        return  cbPcOrganizationService.getOrganization(orgStr);
    }



    @ApiOperation(value = "树形结构", notes = "树形结构查询")
    @GetMapping("/tree")
    public Result findTree() {
        return cbPcOrganizationService.findTree();
    }

    @ApiOperation(value = "前端dhHead所用权限验证版接口")
    @PostMapping("/tree")
    public Result findTreeAuthorize(@RequestBody JSONObject param){
        JSONArray orgSet = param.getJSONArray("orgSet");
        return cbPcOrganizationService.findTreeAuthorize(orgSet);
    }

    @ApiOperation(value = "获取orgId", notes = "从组织机构中获取orgId")
    @GetMapping("/getOrgId/{orgId}")
    public Result getOrgId(@PathVariable("orgId") String orgId) {
        List<CbPcOrganization> data =  cbPcOrganizationService.getOrgId(orgId);
        return Result.ok(data);
    }

    @ApiOperation(value = "获取orgId", notes = "通过组织架构id和等级获取所属id")
    @GetMapping("/findAllIdByIdLevel")
    public Result findAllIdByIdLevel(
            @RequestParam("orgIdValue") @ApiParam("输入组织架构id") String orgIdValue,
            @RequestParam("orgLevel") @ApiParam("输入等级") int orgLevel
    ){
        return Result.ok(cbPcOrganizationService.findAllIdByIdLevel(orgIdValue, orgLevel));
    }


    @ApiOperation(value = "getOrgByOrgName(通过组织架构名称查询组织架构id)")
    @GetMapping("/getOrgByOrgName")
    public Result getOrgByOrgName(
            @RequestParam("orgName") @ApiParam("输入组织架构名称") String orgName) {
        return cbPcOrganizationService.getOrgByOrgName(orgName);
    }

    @RequestMapping("/getAllOrgs")
    public Result getAllOrganizations(){
        return Result.ok(cbPcOrganizationService.list());
    }

    @RequestMapping("/getOrgById/{id}")
    public Result getOrgById(@PathVariable("id") String id){
        return Result.ok(cbPcOrganizationService.getOrgById(id));
    }

    @GetMapping("/getWellIdByOrgId")
    public Result getWellIdByOrgId(@RequestParam("orgId")  String orgId){
        return Result.ok(cbPcOrganizationService.getWellIdByOrgId(orgId));
    }

    @GetMapping("/getOrgByWellId/{wellId}")
    public Result getOrgByWellId(@PathVariable("wellId") String wellId){
        return Result.ok(cbPcOrganizationService.getOrgByWellId(wellId));
    }

    @GetMapping("/getMaxLevelOrgId")
    public Result getMaxLevelOrgId(){
        return Result.ok(cbPcOrganizationService.getMaxLevelOrgId());
    }

    @GetMapping("/getWellIdsByOrgIds")
    public Result getWellIdsByOrgIds(@RequestParam List<String> orgIds){
        return Result.ok(cbPcOrganizationService.getWellIdsByOrgIds(orgIds));
    }

    @GetMapping("/getOrgInfoByWellID")
    public Result getOrgInfoByWellID(@RequestParam String wellId){
        return Result.ok(cbPcOrganizationService.getOrgInfoByWellID(wellId));
    }

    @GetMapping("/getOrgIdByOrgLevel")
    public Result getOrgIdByOrgLevel(){
        return Result.ok(cbPcOrganizationService.getOrgIdByOrgLevel());
    }

    @GetMapping("/getWellIdAndOrgIdLevelOne")
    public Result  getWellIdAndOrgIdLevelOne(@RequestParam List<String> orgIds){
        return Result.ok(cbPcOrganizationService.getWellIdAndOrgIdLevelOne(orgIds));
    }

    @GetMapping("/getOrgByName")
    public Result getOrgByName(@RequestParam String orgid){
        return Result.ok(cbPcOrganizationService.getOrgByName(orgid));
    }

    @GetMapping("/findAllIdById")
    public Result findAllIdById(@RequestParam String orgid){
        return Result.ok(cbPcOrganizationService.getOrgByName(orgid));
    }






}

