package com.gct.centralbase.controller;


import com.alibaba.fastjson.JSONArray;
import com.gct.centralbase.service.CbCdWellSourceService;
import com.gct.centralbase.service.CbPcWellSeriesInfoService;
import com.gct.common.core.result.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * <p>
 * 单井基础信息 前端控制器
 * </p>
 *
 * @author wsy
 * @since 2021-01-19
 */
@Api(value = "CbPcOrganizationController", description = "基础数据前端-单井基础信息导航")
@RestController
@RequestMapping("/WellSource")
public class CentralBaseSourceController {

    @Autowired
    private CbCdWellSourceService cbCdWellSourceService;


    @ApiOperation(value = "getWellSource(获取单井数据)", notes = "获取单井数据")
    @GetMapping("/WellSource")
    public Result getOrganization(String orgid) {
        return cbCdWellSourceService.getWellSource(orgid);
    }

    @ApiOperation(value = "getJh(根据组织架构Id获取井号数据)", notes = "根据组织架构Id获取井号数据")
    @GetMapping("/jh")
    public Result getJh(@ApiParam(name = "orgid", value = "组织架构Id") @RequestParam(value = "orgid", required = false) String orgid) {
        return cbCdWellSourceService.getjh(orgid);
    }

    @ApiOperation(value = "search wells by orgid ,but only show 10 items", notes = "查询井号")
    @GetMapping("/searchWellsByOrgID")
    public Result searchWellsByOrgID(
            @ApiParam(name = "searchText", value = "查询的部分井号内容") @RequestParam(value = "searchText", required = false) String searchText,
            @ApiParam(name = "orgid", value = "组织架构Id") @RequestParam(value = "orgid", required = false) String orgid
    ) {
        return cbCdWellSourceService.searchWellsByOrgID(searchText, orgid);
    }

    @GetMapping("/searchWellsByOrgIDAuthorize")
    public Result searchWellsByOrgIDAuthorize(
            @RequestParam(required = false) String searchText,
            @RequestParam(required = false,value = "orgid") String orgID,
            @RequestParam String orgRoleStr){
        return cbCdWellSourceService.searchWellsByOrgIDAuthorize(searchText,orgID,JSONArray.parseArray(orgRoleStr));
    }

    @ApiOperation(value = "search wells by orgid ,but only show 10 items,the new func", notes = "查询井号")
    @GetMapping("/searchWellsByOrgID2")
    public Result searchWellsByOrgID2(
            @ApiParam(name = "searchText", value = "查询的部分井号内容") @RequestParam(value = "searchText", required = false) String searchText,
            @ApiParam(name = "orgid", value = "组织架构Id") @RequestParam(value = "orgid", required = false) String orgid
    ) {
        return cbCdWellSourceService.searchWellsByOrgID2(searchText, orgid);
    }

    @ApiOperation(value = "查询对应井id的具体信息", notes = "查询井信息")
    @GetMapping("/getWellSource")
    public Result getWellSource(@ApiParam(value = "井号")@RequestParam String wellId){
        return Result.ok(cbCdWellSourceService.getByWellId(wellId));
    }

    @ApiOperation(value = "查询对应井所属区域查询对应井信息", notes = "查询区域井信息")
    @GetMapping("/getWellSourceByStationName")
    public Result getWellSourceByStationName(@ApiParam(value = "站名")@RequestParam(value = "stationName", required = false) String stationName,
                                             @ApiParam(value = "区块名")@RequestParam(value = "producingAreaName", required = false) String producingAreaName){
        return cbCdWellSourceService.getWellSourceByStationName(stationName,producingAreaName);
    }

    @GetMapping("/getWellSourceByWellIdList")
    public Result getWellSource(@RequestParam List<String> wellId){
        return cbCdWellSourceService.getWellSourceByWellIdList(wellId);
    }
    @PostMapping("/getWellInfoByList")
    public Result getWellInfoByList(@RequestBody List<String> wellId){
        return Result.ok(cbCdWellSourceService.getWellInfoByList(wellId));
    }
}

