package com.gct.centralbase.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gct.centralbase.entity.CbCdWellSource;
import com.gct.centralbase.entity.CbPcOrganization;
import com.gct.centralbase.mapper.CbCdWellSourceMapper;
import com.gct.centralbase.service.CbCdWellSourceService;
import com.gct.centralbase.service.CbPcOrganizationService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gct.common.core.result.Result;
import com.gct.common.util.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 单井基础信息 服务实现类
 * </p>
 *
 * @author wsy
 * @since 2021-01-19
 */
@Service
public class CbCdWellSourceServiceImpl extends ServiceImpl<CbCdWellSourceMapper, CbCdWellSource> implements CbCdWellSourceService {


    @Autowired
    private CbPcOrganizationService cbPcOrganizationService;


    /**
     * 获取井号数据
     *
     * @param orgid
     * @return
     */
    @Override
    public Result getWellSource(String orgid) {
        QueryWrapper<CbCdWellSource> queryWrapper = new QueryWrapper<>();
        QueryWrapper<CbCdWellSource> wrapper = queryWrapper.select(
                "well_id",
                "well_common_name",
                "well_legal_name",
                "reason",
                "well_purpose",
                "p_start_prod_stitum",
                "target_formation",
                "spud_date",
                "org_id",
                "struct_id",
                "oilfield_id",
                "exp_area_id",
                "group_unit_id",
                "station_id",
                "geo_latitude",
                "geo_longitude",
                "geo_offset_reference",
                "geo_offset_east",
                "geo_offset_north",
                "remarks",
                "is_deviated",
                "end_status",
                "completion_name",
                "PRODUCING_AREA_name",
                "top_tvd",
                "bottom_tvd",
                "completion_id").eq("org_id", orgid);
        List<Map<String, Object>> maps = baseMapper.selectMaps(wrapper);
        return Result.ok(maps);
    }


    /**
     * 根据组织架构Id获取井号数据
     *
     * @param orgid
     * @return
     */
    @Override
    public Result getjh(String orgid) {
        QueryWrapper<CbCdWellSource> wrapper = new QueryWrapper<>();
        if (orgid == null)
        {
            wrapper.select("well_common_name", "well_id");
        }
        else {
            if (orgid.contains("["))
                orgid = JSONObject.parseArray(orgid).get(1).toString();
            List<String> orgidS = null;
            if (orgid != null && !"".equals(orgid)) {
                orgidS = cbPcOrganizationService.findAllIdById(orgid);
            }
            if (orgidS == null || orgid.isEmpty()) return Result.error("not found");

            wrapper.select("well_common_name", "well_id").in("org_id", orgidS);

        }

        List<Map<String, Object>> mps = baseMapper.selectMaps(wrapper);
        //region deal fe -  data formatter
        mps = formatDataForFELoad(mps);
        //endregion
        return Result.ok(mps);
    }

    private List<Map<String, Object>> formatDataForFELoad(List<Map<String, Object>> mps) {
        List<Map<String, Object>> re = new ArrayList<>();
        if (mps.isEmpty() || mps.size() == 0) return re;
        for (Map<String, Object> mp : mps) {
            HashMap<String, Object> tmp = new HashMap<String, Object>();
            tmp.put("id", String.class.cast(mp.get("well_id")));
            tmp.put("value", String.class.cast(mp.get("well_common_name")));
            re.add(tmp);
        }
        return re;
    }

    /*
     * 需求：
     * orgId 为空时，走权限验证过滤无权限的井
     * orgId 不为空时 且 searchText为空时，搜索 orgId下的井
     * orgId 不为空时 且 searchText不为空，走权限验证过滤无权限的井，按searchText搜索
     * orgId 为空 searchText 不为空 使用searchTest 搜索
     * orgId 为空 serarchText 为空 搜索权限下的所有井
     * orgId 不为空 searchText 不为空 按照searchText 搜索
     * orgId 不为空 searchText 为空 按照 orgId 最小的搜索
     */
    public Result searchWellsByOrgIDAuthorize(String searchText,String orgID,JSONArray orgRoleStr){ //权限控制下的井号搜索接口
        searchText = searchText.trim().replaceAll(" ","");
        if (!StringUtil.isBlankOrEmpty(searchText)){
            //直接按照 传过来的井号查询
            return searchWellsByOrgIDorgRoleStr(searchText,orgRoleStr);
        }else {
            if (!StringUtil.isBlankOrEmpty(orgID)){
                //查询该用户权限下orgId 最小所对应的所有井
                JSONArray jsonArray = new JSONArray();
                jsonArray.add(orgID);
                return searchWellsByOrgIDorgRoleStr(null, jsonArray);
            } else if (orgRoleStr.size()>0){
                return searchWellsByOrgIDorgRoleStr(null,orgRoleStr);
                //查询该用户权限下的所有井
            }else {
                return null;
            }
        }
    }

    private Result searchWellsByOrgIDorgRoleStr(String searchText, JSONArray orgRoleStr){
        String parAndSonOrg = cbPcOrganizationService.getParAndSonOrg(orgRoleStr);
        Set<Object> orgSet = new HashSet<>(JSONArray.parseArray(parAndSonOrg));
        QueryWrapper<CbPcOrganization> queryWrapper = new QueryWrapper<CbPcOrganization>()
                .select("org_id").eq("org_level",3);
        List<CbPcOrganization> list = cbPcOrganizationService.list(queryWrapper);
        List<CbPcOrganization> collect = list.stream().filter(item -> orgSet.contains(item.getOrgId())).collect(Collectors.toList());
        List<String> wellIdList = new ArrayList<>();
        collect.forEach(x->{wellIdList.add(x.getOrgId());});
        QueryWrapper<CbCdWellSource> wrapper = new QueryWrapper<>();
        wrapper.select("well_common_name", "well_id").in("org_id", wellIdList)
                .orderByAsc("well_common_name");
        if (!StringUtil.isBlankOrEmpty(searchText))wrapper = wrapper.like("well_common_name",searchText);
        List<Map<String, Object>> mps = baseMapper.selectMaps(wrapper);
        mps = formatDataForFELoad(mps);
        return Result.ok(mps);
    }

    @Override
    @Deprecated
    public Result searchWellsByOrgID(String searchText, String orgID) {
        if (orgID == null || orgID.isEmpty()) orgID = "0";
        if (orgID.contains("["))
            orgID = JSONObject.parseArray(orgID).get(1).toString();
        List<String> orgidS = null;
        if (orgID != null && !"".equals(orgID)) {
            orgidS = cbPcOrganizationService.findAllIdById(orgID);
        }
        if (orgidS == null || orgidS.isEmpty()) return Result.error("not found");
        QueryWrapper<CbCdWellSource> wrapper = new QueryWrapper<>();
        wrapper.select("well_common_name", "well_id").in("org_id", orgidS).like("well_common_name", searchText)
                .orderByAsc("well_common_name");
        List<Map<String, Object>> mps = baseMapper.selectMaps(wrapper);
        //region deal fe -  data formatter
        mps = formatDataForFELoad(mps);
        //endregion
        return Result.ok(mps);
    }

    @Override
    public Result searchWellsByOrgID2(String seachText, String orgID) {
        JSONArray keys = JSONArray.parseArray(orgID);
        if (keys.size() < 2) return Result.ok(new Object[]{});
        Integer level = Integer.class.cast(keys.get(0));
        String orgid = keys.get(1).toString();
        List<String> orgidS = null;
        if (orgID != null && !"".equals(orgID)) {
            orgidS = cbPcOrganizationService.findSubOrgIDWithMaxLevel(orgid, level);
        }
        QueryWrapper<CbCdWellSource> wrapper = new QueryWrapper<>();
        wrapper.select("well_common_name", "well_id").in("org_id", orgidS).like("well_common_name", seachText)
                .orderByAsc("well_common_name").last("limit 10");
        List<Map<String, Object>> mps = baseMapper.selectMaps(wrapper);
        //region deal fe -  data formatter
        mps = formatDataForFELoad(mps);
        //endregion
        return Result.ok(mps);
    }

    @Override
    public CbCdWellSource getByWellId(String wellId) {
        return baseMapper.selectOne(new QueryWrapper<CbCdWellSource>().eq("well_id", wellId));
    }



    public Result getWellSourceByStationName(String stationName,String producingAreaName)
    {
        QueryWrapper<CbCdWellSource> wrapper=null;
        if (stationName==null)
        {
            wrapper.eq("station_name",stationName);
        }
        else {
            wrapper.eq("producing_area_name",producingAreaName);
        }


        return Result.ok(baseMapper.selectList(wrapper));
    }

   /* @Override
    public Result getWellInfo(Integer curPage,Integer pageSize) {
        System.out.println(curPage+","+pageSize);
        IPage<CbCdWellSource> page = new Page<>(curPage,pageSize);
        List<CbCdWellSource> wellInfoList = baseMapper.selectPage(page, new QueryWrapper<CbCdWellSource>()).getRecords();
        return Result.ok(wellInfoList);
    }*/

    @Override
    public Result getWellInfo(Integer curPage, Integer pageSize) {
        int minLevel = 1;
        Integer maxLevel = cbPcOrganizationService.getMaxLevelFromOrg();
        if (maxLevel == null) return null;
        String sql = "select t.org_id org_id,t.well_id,t.well_common_name well_name,t.spud_date,t2.org_id level3,t2.org_name level_name3,t2.parent_id level3_parent_id from centralbase.cb_cd_well_source t left join centralbase.cb_pc_organization t2 on t.org_id=t2.org_id";
        for (int i = maxLevel - 1; i >= minLevel; i--) {
            sql = " select  m" + i + ".*, m" + i + "2.org_id level" + i + ",m" + i + "2.org_name level_name" + i + ",m" + i + "2.parent_id level" + i + "_parent_id from (" + sql + ") m" + i + " left join centralbase.cb_pc_organization m" + i + "2 on m" + i + ".level" + (i + 1) + "_parent_id=m" + i + "2.org_id";
        }
        Page<Map<String, Object>> page = baseMapper.getAllWellSourceList(new Page<Map<String, Object>>(curPage, pageSize), sql);
        return Result.ok(page);
    }

    /**
     * 查询所有井
     * @return
     */
    @Override
    public Result getjhAll()
    {
        return Result.ok(baseMapper.getjhAll());
    }

    /**
     * 查询所有井用于分析误差率
     * @return
     */
    @Override
    public Result getjhAllYieldError()
    {
        return Result.ok(baseMapper.getjhAllYieldError());
    }

    /**
     * 查询所有井
     * @return
     */
    @Override
    public Result getjhAllConent()
    {
        return Result.ok(baseMapper.getjhAllConent());
    }





    /**
     * 根据组织架构id查询井
     * @param orgid
     * @return
     */
    @Override
    public Result getjhAllByorgid(String orgid)
    {
        return Result.ok(baseMapper.getjhAllByorgid(orgid));
    }



    /**
     * 根据井号查询稠油
     * @param wellName
     * @return
     */
    public Result getByWellId_Heavy(String wellName)
    {
        return Result.ok(baseMapper.getByWellId_Heavy(wellName));
    }

    @Override
    public Result getWellSourceByWellIdList(List<String> wellId) {
        return Result.ok(baseMapper.selectList(new QueryWrapper<CbCdWellSource>().in("well_id",wellId)));
    }

    @Override
    public List<CbCdWellSource> getWellInfoByList(List<String> wellId) {
        return wellId.isEmpty() ? null : baseMapper.selectList(new QueryWrapper<CbCdWellSource>().in("well_id",wellId));
    }
}
