package com.gct.centralbase.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gct.centralbase.entity.CbPcDevWellAttr;
import com.gct.centralbase.mapper.CbPcDevWellAttrMapper;
import com.gct.centralbase.service.CbCdWellSourceService;
import com.gct.centralbase.service.CbPcDevWellAttrService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gct.centralbase.service.CbPcOrganizationService;
import com.gct.common.core.result.CodeMsg;
import com.gct.common.core.result.CodeMsgEnum;
import com.gct.common.core.result.Result;
import com.gct.common.security.util.PasswordUtils;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import springfox.documentation.spring.web.json.Json;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 开发单井属性数据 服务实现类
 * </p>
 *
 * @author wsy
 * @since 2021-08-16
 */
@Service
public class CbPcDevWellAttrServiceImpl extends ServiceImpl<CbPcDevWellAttrMapper, CbPcDevWellAttr> implements CbPcDevWellAttrService {

    @Autowired
    private CbCdWellSourceService cbCdWellSourceService;

    @Autowired
    private CbPcDevWellAttrMapper cbPcDevWellAttrMapper;

    /**
     *根据Id查询单井属性数据
     * @param well_id
     * @return
     */
    @Override
    public Result selectOneByWellId(String well_id){
        CbPcDevWellAttr cbPcDevWellAttr = baseMapper.selectOneByWellId(well_id);
        return Result.ok(cbPcDevWellAttr);
    }

    /**
     *根据组织架构id查询分离器数据
     * @param OrgIds
     * @return
     */
    @Override
    public Result selectOneByOrgid(List<String> OrgIds,String wellId,String dateTime){
        List<CbPcDevWellAttr>  cbPcDevWellAttrs =null;

        if (wellId==null)
        {
            cbPcDevWellAttrs =  baseMapper.selectOneByOrgid(OrgIds,dateTime);
        }
        else {
            cbPcDevWellAttrs =  baseMapper.selectOneByWell(wellId,dateTime);
        }
        return Result.ok(cbPcDevWellAttrs);
    }



    /**
     *根据Id查询单井属性数据
     * @param wellId
     * @return
     */
    @Override
    public Result selectOneByWellIdSeparator(String wellId,String jobSystem){
        CbPcDevWellAttr cbPcDevWellAttr = baseMapper.selectOneByWellIdSeparator(wellId,jobSystem);
        return Result.ok(cbPcDevWellAttr);
    }


    /**
     * 添加单井属性数据
     * @param cbPcDevWellAttr
     * @return
     * author 王森玉
     * time 2021-8-16
     */
    @Override
    @Transactional
    public Result addCbPcDevWellAttr(CbPcDevWellAttr cbPcDevWellAttr) {
        //添加单井属性数据
        if (Objects.isNull(cbPcDevWellAttr.getCoefficient()))return Result.ok("coefficient is null");
         if (this.findCbPcDevWellAttrByDate(cbPcDevWellAttr.getWellId(),cbPcDevWellAttr.getUpdateTime())!=null) { //标定
             deletecbPcDevWellAttr(cbPcDevWellAttr.getWellId(),cbPcDevWellAttr.getUpdateTime());
         }
        int insert = baseMapper.insert(cbPcDevWellAttr);
        return insert==1?Result.create(CodeMsgEnum.ADD_SUCCESS):Result.create(CodeMsgEnum.ADD_FAIL);
    }




    public CbPcDevWellAttr findCbPcDevWellAttrByDate(String wellId,String updateDate) {
        QueryWrapper<CbPcDevWellAttr> queryWrapper = new QueryWrapper<>();   //创建查询queryWrapper
        queryWrapper.eq("well_id", wellId);
        queryWrapper.eq("update_time", updateDate);
        return baseMapper.selectOne(queryWrapper);
    }


    /**
     * 对单井属性配置表进行是否匹配清空
     * @param
     * @param
     * @return
     */
    @Override
    public Result deletecbPcDevWellAttr(String wellid,String UpdateTime)
    {
        return Result.ok(baseMapper.deletecbPcDevWellAttr(wellid,UpdateTime));
    }

    /**
     * 对单井属性配置表进行是否匹配清空
     * @param
     * @param
     * @return
     */
    @Override
    public Result deleteCbPcDevWellAttrLess(String wellid,String updateTime)
    {
        return Result.ok(baseMapper.deleteCbPcDevWellAttrLess(wellid,updateTime));
    }




    /**
     * 修改单井属性信息
     * @param cbPcDevWellAttr
     * @return
     * author 王森玉
     * time 2021-8-16
     */
    public Result updateCbPcDevWellAttr(CbPcDevWellAttr cbPcDevWellAttr)
    {


        int update = baseMapper.updateById(cbPcDevWellAttr);
        return update==1?Result.create(CodeMsgEnum.MODIFY_SUCCESS):Result.create(CodeMsgEnum.MODIFY_FAIL);
    }



    /**
     * 修改单井属性信息（稀油）
     * @param cbPcDevWellAttr
     * @return
     * author 王森玉
     * time 2021-8-16
     */
    public Result updateCbPcDevWellAttrThinoil(CbPcDevWellAttr cbPcDevWellAttr)
    {

        return Result.ok(baseMapper.updateCbPcDevWellAttr(cbPcDevWellAttr.getWellId(),cbPcDevWellAttr.getOilProductsSorting(),cbPcDevWellAttr.getMarkedParmExplain(),
                cbPcDevWellAttr.getIsmatching(),cbPcDevWellAttr.getCoefficient(),cbPcDevWellAttr.getUpdateTime(),cbPcDevWellAttr.getMarkedBy()));
    }




    /**
     * 根据参数条件查询单井属性数据
     * author 王森玉
     * time 2021-8-16
     * @param pageNum
     * @param pageSize
     * @param org_id
     * @param well_id
     * @param oil_products_sorting
     * @param test_type
     * @param start_prod_stitum
     * @param oil_production_method
     * @param injection_method
     * @param is_deviated
     * @param beginDate
     * @param endDate
     * @return
     */
    public Result findAllCbPcDevWellAttr( Integer pageNum, Integer pageSize,String org_id,String well_id,String oil_products_sorting, String test_type, String start_prod_stitum, String oil_production_method, String injection_method, String is_deviated, String beginDate,String endDate)
    {
        String org_id1 ="";
        List<String> well_id1=new ArrayList<>();
        String oil_products_sorting1 ="";
        String test_type1 ="";
        String start_prod_stitum1 ="";
        String oil_production_method1 ="";
        String injection_method1 ="";
        String is_deviated1 ="";

        List<String> stringList = new ArrayList<>();
        Result jh2 =cbCdWellSourceService.getjh(org_id);
        JSONArray list =(JSONArray)JSONObject.toJSON(jh2.getData());
        for (int i = 0; i < list.size(); i++) {
            stringList.add( ((JSONObject)list.get(i)).getString("id"));
        }

        if (well_id == null || well_id.length() == 0) {
            well_id1 = stringList;
        }
        else {
            well_id1.add(well_id);
        }


        if (!"全部".equals(oil_products_sorting) && !"".equals(oil_products_sorting))
        {
            oil_products_sorting1 += oil_products_sorting;
        }
        else
        {
            oil_products_sorting1 +="";
        }
        if (!"全部".equals(test_type) && !"".equals(test_type))
        {
            test_type1 += test_type;
        }
        else
        {
            test_type1 +="";
        }
        if (!"全部".equals(start_prod_stitum) && !"".equals(start_prod_stitum))
        {
            start_prod_stitum1 += start_prod_stitum;
        }
        else
        {
            start_prod_stitum1 +="";
        }
        if (!"全部".equals(oil_production_method) && !"".equals(oil_production_method))
        {
            oil_production_method1 += oil_production_method;
        }
        else {
            oil_production_method1 += "";
        }
        if (!"全部".equals(injection_method) && !"".equals(injection_method))
        {
            injection_method1 += injection_method;
        }
        else
        {
            injection_method1 += "";
        }
        if (!"全部".equals(is_deviated) && !"".equals(is_deviated))
        {
            is_deviated1 += is_deviated;
        }
        else
        {
            is_deviated1 += "";
        }


        Page<CbPcDevWellAttr> paramPage = new Page<>(pageNum, pageSize);
        paramPage.offset();
        paramPage.addOrder(OrderItem.desc("update_time"));

        IPage<CbPcDevWellAttr> page = null;
        page = baseMapper.selectCbPcDevWellAttrList
                (paramPage,well_id1,oil_products_sorting1, test_type1, start_prod_stitum1,oil_production_method1,injection_method1,is_deviated1,beginDate,endDate);
        for (CbPcDevWellAttr record : page.getRecords()) {

            String markedParmExplain = record.getMarkedParmExplain();


            if (markedParmExplain == null || markedParmExplain.length() <= 0) {
                record.setExplain("无");
            }
            else if (markedParmExplain.equals("null"))
            {
                record.setExplain("无");
            }
            else {
                JSONObject jsonObject = JSONObject.parseObject(markedParmExplain);
                String explain = jsonObject.getString("explain");
                if (explain.equals("null"))
                {
                    record.setExplain("无");
                }
                else {
                    record.setExplain(explain);
                }

            }

        }



        return Result.ok(page);

    }

    /**
     * 修改膨胀系数
     * @param wellid
     * @param pzxh
     * @return
     */
    @Override
    public Result updatePzxs(String wellid,double pzxh)
    {
        return Result.ok(baseMapper.updatePzxs(wellid,pzxh));
    }

    /**
     * 根据井号，更新时间修改膨胀系数
     * @param wellid
     * @param upateTime
     * @return
     */
    @Override
    public Result updateCoefficientByJh(String wellid,String upateTime,double xs)
    {
        return Result.ok(baseMapper.updateCoefficientByJh(wellid,upateTime,xs));
    }



    /**
     * 修改是否需要自动匹配
     * @param wellid
     * @param sfpp
     * @return
     */
    @Override
    public Result updateis(String wellid,String sfpp,String dateTime,String fl)
    {
        return Result.ok(baseMapper.updateis(wellid,sfpp,dateTime,fl));
    }



    /**
     * 对单井属性配置表进行是否匹配清空
     * @param
     * @param
     * @return
     */
    @Override
    public Result updateisempty(String wellid)
    {
        return Result.ok(baseMapper.updateisempty(wellid));
    }

    /**
     * 获取单井属性柱状图
     * @return
     */
    @Override
    public Result find_CbPcDevWellAttr_tj()
    {
        QueryWrapper<CbPcDevWellAttr> queryWrapper = new QueryWrapper<>();

        QueryWrapper<CbPcDevWellAttr> wrapper = queryWrapper.select("(case when ismatching='是' then '匹配'  when ismatching='否' then '不匹配'\n" +
                        "else '其它' end) as ismatching , count(1) as iscount")
                .groupBy("ismatching");

        List<CbPcDevWellAttr>  cbPcDevWellAttrclass = cbPcDevWellAttrMapper.selectList(wrapper);
        //封装结果集
        LinkedHashMap<String, Object> resultMap = new LinkedHashMap<>();
        //eCharts图表
        LinkedHashMap<String, Object> eCharts = new LinkedHashMap<>();
        //横坐标
        List<String> xismatching = new ArrayList<>();
        //累计
        LinkedHashMap<String, Object> eChartscount = new LinkedHashMap<>();
        List<Double> eChartscountdata = new ArrayList<>();


        if (cbPcDevWellAttrclass!=null)
        {
            //遍历
            cbPcDevWellAttrclass.forEach((cbPcDevWellAttr) -> {
                double count =0;
                if(cbPcDevWellAttr.getIscount() ==null || cbPcDevWellAttr.getIscount().equals(""))
                {
                    count = 0;
                }
                else {
                    count = cbPcDevWellAttr.getIscount();
                }
                eChartscountdata.add(count);//类型次数


                xismatching.add(cbPcDevWellAttr.getIsmatching());
            });
        }

        //组装数据
        eChartscount.put("name", "单井属性各个类型数量统计");
        eChartscount.put("data", eChartscountdata);
        eCharts.put("xAxis", xismatching);
        eCharts.put("series", new Object[]{eChartscount});
        resultMap.put("eCharts", eCharts);
        return Result.ok(resultMap);
    }

    /**
     * 根据单井属性类型获取对应的详细信息（柱状图）
     * @return
     */
    @Override
    public Result find_CbPcDevWellAttr_tjxx(String ismatching)
    {
        QueryWrapper<CbPcDevWellAttr> queryWrapper = new QueryWrapper<>();
        QueryWrapper<CbPcDevWellAttr> wrapper;
        String ismatchingdata="";

        if(ismatching.equals("其它"))
        {
            wrapper = queryWrapper.select("update_time,count(1) as iscount")
                    .isNull("ismatching")
                    .groupBy("update_time")
                    .orderByDesc("update_time");
        }
        else {
            if (ismatching.equals("匹配"))
            {
                ismatchingdata ="是";
            }else if (ismatching.equals("不匹配"))
            {
                ismatchingdata ="否";
            }
            wrapper = queryWrapper.select("update_time,count(1) as iscount")
                    .eq("ismatching",ismatchingdata)
                    .groupBy("update_time")
                    .orderByDesc("update_time");
        }

        List<CbPcDevWellAttr>  cbPcDevWellAttrclass = cbPcDevWellAttrMapper.selectList(wrapper);
        //封装结果集
        LinkedHashMap<String, Object> resultMap = new LinkedHashMap<>();
        //eCharts图表
        LinkedHashMap<String, Object> eCharts = new LinkedHashMap<>();
        //横坐标
        List<String> xdate = new ArrayList<>();
        //累计次数
        LinkedHashMap<String, Object> eChartscount = new LinkedHashMap<>();
        List<Double> eChartscountdata = new ArrayList<>();


        if (cbPcDevWellAttrclass!=null)
        {
            //遍历
            cbPcDevWellAttrclass.forEach((cbPcDevWellAttr) -> {
                double count =0;
                if(cbPcDevWellAttr.getIscount() ==null || cbPcDevWellAttr.getIscount().equals(""))
                {
                    count = 0;
                }
                else {
                    count = cbPcDevWellAttr.getIscount();
                }
                eChartscountdata.add(count);//类型次数


                String UpdateTime =cbPcDevWellAttr.getUpdateTime().length()>10? cbPcDevWellAttr.getUpdateTime().substring(0,10):cbPcDevWellAttr.getUpdateTime();
                xdate.add(UpdateTime);
            });
        }

        //组装数据
        eChartscount.put("name", "单井系数详细数据");
        eChartscount.put("data", eChartscountdata);
        eCharts.put("xAxis", xdate);
        eCharts.put("series", new Object[]{eChartscount});
        resultMap.put("eCharts", eCharts);
        return Result.ok(resultMap);
    }

    @Override
    public CbPcDevWellAttr getCbPcDevWellAttrMaxUpdateTime(String wellId,String realDate,String flag)
    {

        QueryWrapper<CbPcDevWellAttr> queryWrapper = new QueryWrapper<>();
        if (flag.equals("1"))
        {
            queryWrapper.select("max(update_time) update_time")
                    .eq("well_id",wellId)
                    .ne("update_time",realDate);
        }
        else if (flag.equals("2")){
            queryWrapper.select("max(update_time) update_time")
                    .eq("well_id",wellId)
                    .lt("update_time",realDate)
                    .like("marked_parm_explain","null");
        }
        else if (flag.equals("3")){
            queryWrapper.select("coefficient","marked_by","marked_parm_explain")
                    .eq("well_id",wellId)
                    .eq("update_time",realDate);
        }

        else if (flag.equals("4")){
            queryWrapper.select("max(update_time)")
                    .eq("well_id",wellId)
                    .notLike("marked_parm_explain","标定");
        }


        return cbPcDevWellAttrMapper.selectOne(queryWrapper);
    }


    /**
     * 根据井号查询单井属性数据(集合)
     * @param wellId
     * @return
     */
    @Override
    public List<CbPcDevWellAttr> selectOneByWellIdList(String wellId)
    {
        QueryWrapper<CbPcDevWellAttr> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("well_id",wellId);
        queryWrapper.orderByDesc("update_time");
        queryWrapper.last("limit 1");//查询一条数据
        return baseMapper.selectList(queryWrapper);
    }

    /**
     * 根据井号时间查询单井属性数据(集合)
     * @param wellId
     * @return
     */

    @Override
    public List<CbPcDevWellAttr> selectOneByWellIdTimeList(String wellId,String updateTime)
    {
        QueryWrapper<CbPcDevWellAttr> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("well_id",wellId);
        queryWrapper.le("update_time",updateTime);
        queryWrapper.orderByDesc("update_time");
        queryWrapper.last("limit 1");//查询一条数据
        return baseMapper.selectList(queryWrapper);
    }



}
