package com.gct.centralbase.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gct.centralbase.entity.CbCdWellSource;
import com.gct.centralbase.entity.CbPcDevWellAttr;
import com.gct.centralbase.entity.CbPcProWellMeterTeam;
import com.gct.centralbase.mapper.CbPcProWellMeterTeamMapper;
import com.gct.centralbase.service.CbCdWellSourceService;
import com.gct.centralbase.service.CbPcProWellMeterTeamService;
import com.gct.common.core.result.Result;
import com.gct.common.util.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;

import java.sql.Timestamp;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

/**
 * <p>
 * 人工计产数据 服务类
 * </p>
 *
 * @author wsy
 * @since 2021-09-10
 */
@Service
public class CbPcProWellMeterTeamServiceImpl extends ServiceImpl<CbPcProWellMeterTeamMapper, CbPcProWellMeterTeam> implements CbPcProWellMeterTeamService {

    @Autowired
    CbPcProWellMeterTeamMapper cbPcProWellMeterTeamMapper;

    @Autowired
    CbCdWellSourceService cbCdWellSourceService;

    /**
     * 针对量液服务获取计产数据
     * @param wellName
     * @param dateTime
     * @return
     */
    @Override
    public Result getWellMeterTeamForCalProduction(String wellName, String dateTime){
        CbPcProWellMeterTeam  cbPcProWellMeterTeam = cbPcProWellMeterTeamMapper.getWellMeterTeamForCalProduction(wellName,dateTime);
        return Result.ok(cbPcProWellMeterTeam);
    }

    @Override
    public Page<CbPcProWellMeterTeam> getList(JSONObject jsonObject) {
        String wellId = jsonObject.getString("wellId");
        int cuttentPage = jsonObject.getIntValue("cuttentPage");
        int pageSize = jsonObject.getIntValue("pageSize");
        JSONArray ids = jsonObject.getJSONArray("ids");
        LambdaQueryWrapper<CbPcProWellMeterTeam> wrapper = new LambdaQueryWrapper<>();
        if (Objects.nonNull(ids)){
            wrapper = wrapper.in(CbPcProWellMeterTeam::getWellId,ids);
        }
        if (!StringUtil.isBlankOrEmpty(wellId)){
            wrapper = wrapper.eq(CbPcProWellMeterTeam::getWellId,wellId);
        }
        wrapper= wrapper.orderByAsc(CbPcProWellMeterTeam::getWellId);
        Page<CbPcProWellMeterTeam> page = baseMapper.selectPage(new Page<>(cuttentPage, pageSize), wrapper);
        List<CbPcProWellMeterTeam> teams =page.getRecords() ;
        if (!teams.isEmpty()) {
            List<String> stringList = new ArrayList<>();
            teams.forEach(x -> stringList.add(x.getWellId()));
            List<CbCdWellSource> wellInfoByList = cbCdWellSourceService.getWellInfoByList(stringList);
            wellInfoByList.forEach(x->{
                teams.forEach(y->{
                    if (x.getWellId().equals(y.getWellId())){
                        String[] split = x.getStationId().split("@");
                        y.setZyq(split[0]);
                        y.setJq(split[1]);
                        y.setZd(split[2]);
                    }
                });
            });
        }
        return page;
    }

    @Override
    public boolean saveData(CbPcProWellMeterTeam cbPcProWellMeterTeam) {

        if (this.findCbcbPcProWellMeterTeam(cbPcProWellMeterTeam.getWellId(),cbPcProWellMeterTeam.getStartProdDate()).size()>0) { //标定
            deletecbPcProWellMeterTeam(cbPcProWellMeterTeam.getWellId(),cbPcProWellMeterTeam.getStartProdDate());
        }

        String username= SecurityContextHolder.getContext().getAuthentication().getName();
        cbPcProWellMeterTeam.setCreateUserId(username);
        cbPcProWellMeterTeam.setUpdateUserId(username);
        cbPcProWellMeterTeam.setCreateDate(LocalDateTime.now());
        cbPcProWellMeterTeam.setUpdateDate(LocalDateTime.now());



        return baseMapper.insert(cbPcProWellMeterTeam) > 0;
    }

    @Override
    public List<CbPcProWellMeterTeam> findCbcbPcProWellMeterTeam(String wellId,LocalDateTime prodDate) {
        QueryWrapper<CbPcProWellMeterTeam> queryWrapper = new QueryWrapper<>();   //创建查询queryWrapper
        queryWrapper.eq("well_id", wellId);
        queryWrapper.eq("start_prod_date", prodDate);
        return baseMapper.selectList(queryWrapper);
    }

    @Override
    public Result deletecbPcProWellMeterTeam(String wellid,LocalDateTime prodDate)
    {
        return Result.ok(baseMapper.deletecbPcProWellMeterTeam(wellid,prodDate));
    }

}
