package com.gct.centralbase.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gct.centralbase.entity.CbPcDevWellAttr;
import com.gct.centralbase.entity.CbPcProWellboreStatusDaily;
import com.gct.centralbase.entity.CbPcProWellboreVolDaily;
import com.gct.centralbase.mapper.CbPcProWellboreStatusDailyMapper;
import com.gct.centralbase.mapper.CbPcProWellboreVolDailyMapper;
import com.gct.centralbase.service.CbPcOrganizationService;
import com.gct.centralbase.service.CbPcProWellboreVolDailyService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gct.common.core.result.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.sql.Timestamp;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

/**
 * <p>
 * 采出井井筒生产日数据 服务实现类
 * </p>
 *
 * @author wsy
 * @since 2021-01-19
 */
@Service
public class CbPcProWellboreVolDailyServiceImpl extends ServiceImpl<CbPcProWellboreVolDailyMapper, CbPcProWellboreVolDaily> implements CbPcProWellboreVolDailyService {

    @Autowired
    private CbPcProWellboreVolDailyMapper cbPcProWellboreVolDailyMapper;

    @Autowired
    private CbPcProWellboreStatusDailyMapper cbPcProWellboreStatusDailyMapper;

    @Autowired
    private CbPcOrganizationService cbPcOrganizationService;

    private DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");

    @Override
    public Result getYieldCurve(String wellId, String beginDate, String endDate) {
        //生产曲线查询  液量、油量、含水
        LocalDateTime begin = LocalDateTime.parse(beginDate.length() == 10 ? beginDate + " 00:00:00" : beginDate, df);
        LocalDateTime end = LocalDateTime.parse(endDate.length() == 10 ? endDate + " 00:00:00" : endDate, df);
        QueryWrapper<CbPcProWellboreVolDaily> queryWrapper = new QueryWrapper<>();
        queryWrapper.select(
                "prod_date",
                "liq_prod_daily",
                "oil_prod_daily",
                "water_cut")
                .ge("prod_date", begin)
                .le("prod_date", end)
                .eq("well_id", wellId)
                .orderByAsc("prod_date");
        //动液面查询  动液面
        QueryWrapper<CbPcProWellboreStatusDaily> queryWrapper1 = new QueryWrapper<>();
        QueryWrapper<CbPcProWellboreStatusDaily> wrapper1 = queryWrapper1
                .select("well_id", "prod_date", "start_pump_liq_level","submergence_depth")
                .ge("prod_date", begin)
                .le("prod_date", end)
                .eq("well_id", wellId)
                .orderByAsc("prod_date");
        List<CbPcProWellboreVolDaily> volDailies = cbPcProWellboreVolDailyMapper.selectList(queryWrapper);
        List<CbPcProWellboreStatusDaily> mechDailies = cbPcProWellboreStatusDailyMapper.selectList(wrapper1);
        //封装结果集
        LinkedHashMap<String, Object> resultMap = new LinkedHashMap<>();
        //第一个eCharts图表
        LinkedHashMap<String, Object> eCharts1 = new LinkedHashMap<>();
        //第二个eCharts图表
        LinkedHashMap<String, Object> eCharts2 = new LinkedHashMap<>();
        //横坐标
        List<String> x1Date = new ArrayList<>();
        List<String> x2Date = new ArrayList<>();
        //日产液量
        LinkedHashMap<String, Object> eCharts1Liq = new LinkedHashMap<>();
        List<Double> eCharts1LiqData = new ArrayList<>();
        //日产油量
        LinkedHashMap<String, Object> eCharts1Oil = new LinkedHashMap<>();
        List<Double> eCharts1OilData = new ArrayList<>();
        //含水率
        LinkedHashMap<String, Object> eCharts1Water = new LinkedHashMap<>();
        List<Double> eCharts1WaterData = new ArrayList<>();
        //动液面
        LinkedHashMap<String, Object> eCharts2Level = new LinkedHashMap<>();
        List<Double> eCharts2LevelData = new ArrayList<>();
        //沉没度
        LinkedHashMap<String, Object> eCharts2CMD = new LinkedHashMap<>();
        List<Double> eCharts2CMDData = new ArrayList<>();
        //遍历
        volDailies.forEach((cbPcProWellboreVolDaily) -> {
            eCharts1LiqData.add(cbPcProWellboreVolDaily.getLiqProdDaily());
            eCharts1OilData.add(cbPcProWellboreVolDaily.getOilProdDaily());
            eCharts1WaterData.add(cbPcProWellboreVolDaily.getWaterCut());
            x1Date.add(DateTimeFormatter.ofPattern("yyyy-MM-dd").format(cbPcProWellboreVolDaily.getProdDate()));
        });
        mechDailies.forEach((cbPcProWellboreStatusDaily -> {
            x2Date.add(DateTimeFormatter.ofPattern("yyyy-MM-dd").format(cbPcProWellboreStatusDaily.getProdDate()));
            eCharts2LevelData.add(cbPcProWellboreStatusDaily.getStartPumpLiqLevel());
            eCharts2CMDData.add(cbPcProWellboreStatusDaily.getSubmergenceDepth());
        }));
        //组装数据
        eCharts1Liq.put("name", "日产液量");
        eCharts1Liq.put("data", eCharts1LiqData);
        eCharts1Liq.put("unit", "t");
        eCharts1Oil.put("name", "日产油量");
        eCharts1Oil.put("data", eCharts1OilData);
        eCharts1Oil.put("unit", "t");
        eCharts1Water.put("name", "含水率");
        eCharts1Water.put("data", eCharts1WaterData);
        eCharts1Water.put("unit", "%");
        eCharts1.put("xAxis", x1Date);
        eCharts1.put("series", new Object[]{eCharts1Liq, eCharts1Oil, eCharts1Water});
        eCharts2Level.put("name", "动液面");
        eCharts2Level.put("unit", "m");
        eCharts2Level.put("data", eCharts2LevelData);
        eCharts2CMD.put("name", "沉没度");
        eCharts2CMD.put("unit", "m");
        eCharts2CMD.put("data", eCharts2CMDData);
        eCharts2.put("xAxis", x2Date);
        eCharts2.put("series", new Object[]{eCharts2Level,eCharts2CMD});
        resultMap.put("eCharts1", eCharts1);
        resultMap.put("eCharts2", eCharts2);
        return Result.ok(resultMap);
    }

    @Override
    public Result getHistoryMeasures(Integer pageNum, Integer pageSize, String wellId, String beginDate, String endDate) {
        //分页插件
        Page<Map<String, Object>> paramPage = new Page<>(pageNum, pageSize);
        paramPage.offset();
        paramPage.addOrder(OrderItem.asc("prod_date"));
        LocalDateTime begin = LocalDateTime.parse(beginDate.length() == 10 ? beginDate + " 00:00:00" : beginDate, df);
        LocalDateTime end = LocalDateTime.parse(endDate.length() == 10 ? endDate + " 00:00:00" : endDate, df);
        QueryWrapper<CbPcProWellboreVolDaily> queryWrapper = new QueryWrapper<>();
        queryWrapper.select(
                "well_id",
                "prod_date",
                "stim_content")
                .ge("prod_date", begin)
                .le("prod_date", end)
                .eq("well_id", wellId);
        Page<Map<String, Object>> page = cbPcProWellboreVolDailyMapper.selectMapsPage(paramPage, queryWrapper);
        List<Map<String, Object>> records = page.getRecords();
        records.forEach((map) -> {
            map.put("well_id", map.get("well_id"));
            map.put("prod_date", df.format(((Timestamp) map.get("prod_date")).toLocalDateTime()));
            map.put("stim_content", map.get("stim_content"));
        });
        return Result.ok(page);
    }

    @Override
    public Result getWellBoreVolDaily(String wellId, String prodDate) {
        return Result.ok(cbPcProWellboreVolDailyMapper.getWellBoreVolDaily(wellId, prodDate));
    }

    @Override
    public CbPcProWellboreVolDaily getA2DataForCalVolDaily(String wellName, String dataTime) {
        return cbPcProWellboreVolDailyMapper.getA2DataForCalVolDaily(wellName, dataTime);
    }
    @Override
    public Result getCbPcProWellBoreVolDaily(String prodDate, Integer curPage, Integer pageSize) {
        int minLevel = 1;
        Integer maxLevel = cbPcOrganizationService.getMaxLevelFromOrg();
        if (maxLevel == null) return null;
        String sql = "select t.well_id,t.prod_date,t.prod_time,t.org_id,t2.org_id level3,t2.org_name level_name3,t2.parent_id level3_parent_id " +
                "from (select v.well_id,v.prod_date,v.prod_time,w.org_id " +
                "from centralbase.cb_pc_pro_wellbore_vol_daily v,centralbase.cb_cd_well_source w " +
                "where v.well_id = w.well_id and v.prod_date::date = '"+prodDate+"'::date) t " +
                "left join centralbase.cb_pc_organization t2 on t.org_id=t2.org_id";
        for (int i = maxLevel - 1; i >= minLevel; i--) {
            sql = " select  m" + i + ".*, m" + i + "2.org_id level" + i + ",m" + i + "2.org_name level_name" + i + ",m" + i + "2.parent_id level" + i + "_parent_id from (" + sql + ") m" + i + " left join centralbase.cb_pc_organization m" + i + "2 on m" + i + ".level" + (i + 1) + "_parent_id=m" + i + "2.org_id";
        }
        return Result.ok(cbPcProWellboreVolDailyMapper.getCbPcProWellBoreVolDaily(new Page<>(curPage, pageSize), sql));
    }

    @Override
    public List<CbPcProWellboreVolDaily> getVolDailyListByProdDate(LocalDateTime prodDate,String wellId) {
        return baseMapper.selectList(new LambdaQueryWrapper<CbPcProWellboreVolDaily>()
                .eq(CbPcProWellboreVolDaily::getProdDate,prodDate)
                        .eq(CbPcProWellboreVolDaily::getWellId,wellId)
                .select(CbPcProWellboreVolDaily::getWellId,CbPcProWellboreVolDaily::getProdDate,CbPcProWellboreVolDaily::getLiqProdDaily,
                        CbPcProWellboreVolDaily::getWaterCut));
    }
    @Override
    public Result getCbPcProWellboreVolDailyProdDate(String wellId,String identification,String ProdDate)
    {
        LocalDateTime ProdDateValue =null;

        QueryWrapper<CbPcProWellboreVolDaily> queryWrapper = new QueryWrapper<CbPcProWellboreVolDaily>();
        QueryWrapper<CbPcProWellboreVolDaily> wrapper = null;
        if (identification.equals("1"))
        {
            wrapper = queryWrapper.select("max(prod_date) prod_date")
                    .eq("well_id",wellId)
                    .like("remarks","放气");
        }
        else if (identification.equals("2")) {
             ProdDateValue = LocalDateTime.parse(ProdDate.length() == 10 ? ProdDate + " 00:00:00" : ProdDate, df);
            wrapper = queryWrapper.select("max(prod_date) prod_date")
                    .eq("well_id",wellId)
                    .like("remarks","放气")
                    .ne("prod_date",ProdDateValue);
        }
        else if (identification.equals("3")) {
             ProdDateValue = LocalDateTime.parse(ProdDate.length() == 10 ? ProdDate + " 00:00:00" : ProdDate, df).plusDays(-1);
            wrapper = queryWrapper.select("liq_prod_daily")
                    .eq("well_id",wellId)
                    .eq("prod_date",ProdDateValue);
        }
        else if (identification.equals("4")) {
            ProdDateValue = LocalDateTime.parse(ProdDate.length() == 10 ? ProdDate + " 00:00:00" : ProdDate, df).plusDays(+1);
            wrapper = queryWrapper.select("liq_prod_daily")
                    .eq("well_id",wellId)
                    .eq("prod_date",ProdDateValue);
        }

        CbPcProWellboreVolDaily maps = cbPcProWellboreVolDailyMapper.selectOne(wrapper);

        return Result.ok(maps);
    }


    @Override
    public long hotWashCheck7day(String wellId, String prodDate) {
        CbPcProWellboreVolDaily wellboreVolDaily = baseMapper.selectOne(new QueryWrapper<CbPcProWellboreVolDaily>()
                .eq("well_id", wellId)
                .like("remarks", "%洗井%")
                .le("prod_date::date",LocalDate.parse(prodDate,DateTimeFormatter.ISO_LOCAL_DATE))
                .orderByDesc("prod_date")
                .last("limit 1")
        );
        if (Objects.isNull(wellboreVolDaily))return 1000;
        return LocalDate.parse(prodDate).toEpochDay() - wellboreVolDaily.getProdDate().toLocalDate().toEpochDay();
    }
}
