package com.gct.centralbase.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.gct.centralbase.entity.CbTempWellMechDaily;
import com.gct.centralbase.entity.CbTempWellMechRuntime;
import com.gct.centralbase.mapper.CbTempWellMechDailyMapper;
import com.gct.centralbase.service.CbTempWellMechDailyService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gct.centralbase.service.CbTempWellMechRuntimeService;
import com.gct.common.core.result.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;

/**
 * <p>
 * 机采数据 daily 服务实现类
 * </p>
 *
 * @author wsy
 * @since 2021-01-19
 */
@Service
public class CbTempWellMechDailyServiceImpl extends ServiceImpl<CbTempWellMechDailyMapper, CbTempWellMechDaily> implements CbTempWellMechDailyService {

    @Autowired
    CbTempWellMechRuntimeService cbTempWellMechRuntimeService;

    @Override
    public Result getLoadCurve(String wellId, String beginTime, String endTime) {
        //载荷曲线
        QueryWrapper<CbTempWellMechRuntime> queryWrapper = new QueryWrapper<>();
        DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        LocalDateTime begin = LocalDateTime.parse(beginTime, df);
        LocalDateTime end = LocalDateTime.parse(endTime, df);
        QueryWrapper<CbTempWellMechRuntime> wrapper = queryWrapper.select("prod_date", "susp_max_load", "susp_min_load")
                .ge("prod_date", begin)
                .le("prod_date", end)
                .eq("well_id", wellId)
                .orderByAsc("prod_date");
        List<CbTempWellMechRuntime> mechDailies = cbTempWellMechRuntimeService.list(wrapper);
        //封装结果集
        LinkedHashMap<String, Object> resultMap = new LinkedHashMap<>();
        //横坐标
        List<String> xDate = new ArrayList<>();
        //最大载荷
        LinkedHashMap<String, Object> maxLoad = new LinkedHashMap<>();
        List<Double> maxLoadData = new ArrayList<>();
        //最小载荷
        LinkedHashMap<String, Object> minLoad = new LinkedHashMap<>();
        List<Double> minLoadData = new ArrayList<>();

        //载荷差
        LinkedHashMap<String, Object> loadDifference = new LinkedHashMap<>();
        List<Double> loadDifferenceData = new ArrayList<>();

        java.text.DecimalFormat   dfdata   =new   java.text.DecimalFormat("#.00");

        mechDailies.forEach(cbTempWellMechDaily -> {
            String date = cbTempWellMechDaily.getProdDate().format(df);
            // 去掉时区 T
            xDate.add(date.substring(0,10)+' '+date.substring(11));
            maxLoadData.add(cbTempWellMechDaily.getSuspMaxLoad());
            minLoadData.add(cbTempWellMechDaily.getSuspMinLoad());
            BigDecimal bg = new BigDecimal(cbTempWellMechDaily.getSuspMaxLoad()-cbTempWellMechDaily.getSuspMinLoad());
            loadDifferenceData.add(bg.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue());
        });
        maxLoad.put("name","最大载荷");
        maxLoad.put("data",maxLoadData);
        maxLoad.put("unit","KN");
        minLoad.put("name","最小载荷");
        minLoad.put("data",minLoadData);
        minLoad.put("unit","KN");
        loadDifference.put("name","载荷差");
        loadDifference.put("data",loadDifferenceData);
        loadDifference.put("unit","KN");
        resultMap.put("xAxis",xDate);
        resultMap.put("series",new Object[]{maxLoad,minLoad,loadDifference});
        return Result.ok(resultMap);
    }

    @Override
    public Result getWellMechDaily(String wellId, String prodDate) {
        return Result.ok(baseMapper.getWellMechDaily(wellId,prodDate));
    }

}
