package com.gct.centralbase.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gct.centralbase.entity.CbCdWellSource;
import com.gct.centralbase.entity.SysAccessWellControl;
import com.gct.centralbase.entity.Vo.AccessWellExcelVo;
import com.gct.centralbase.entity.Vo.AccessWellInfoVo;
import com.gct.centralbase.mapper.CbCdWellSourceMapper;
import com.gct.centralbase.mapper.SysAccessWellControlMapper;
import com.gct.centralbase.service.CbCdWellSourceService;
import com.gct.centralbase.service.CbPcOrganizationService;
import com.gct.centralbase.service.SysAccessWellControlService;
import com.gct.common.util.DateTimeUtil;
import com.gct.common.util.StringUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * class name: SysAccessWellServiceImpl
 *
 * @author gxt
 * @version 1.0
 * @since 2021/9/22 上午11:15 周三
 */
@Service
public class SysAccessWellControlServiceImpl extends ServiceImpl<SysAccessWellControlMapper, SysAccessWellControl> implements SysAccessWellControlService {

    @Autowired
    SysAccessWellControlMapper accessWellMapper;
    @Autowired
    CbPcOrganizationService cbPcOrganizationService;
    @Autowired
    CbCdWellSourceMapper cbCdWellSourceMapper;
    @Autowired
    CbCdWellSourceService cbCdWellSourceService;

    @Override
    public Object getAccessWellInfo(AccessWellInfoVo accessWellInfoVo) {
        if (Objects.isNull(accessWellInfoVo)) return new ArrayList<>();
        QueryWrapper<SysAccessWellControl> wrapper = new QueryWrapper<>();
        IPage<Map<String, Object>> page = new Page<Map<String, Object>>();
        page.setCurrent(accessWellInfoVo.getCuttentPage());
        page.setSize(accessWellInfoVo.getPageSize());
        if (!StringUtil.isBlankOrEmpty(accessWellInfoVo.getWellId())) {
            wrapper = wrapper.eq("well_id",accessWellInfoVo.getWellId());
        }else {
            if (StringUtil.isBlankOrEmpty(accessWellInfoVo.getOrgId())) accessWellInfoVo.setOrgId("[0,0,3]");
            Object data = cbCdWellSourceService.getjh(accessWellInfoVo.getOrgId()).getData();
            JSONArray jsonArray = (JSONArray) JSONArray.toJSON(data);
            List<Object> id = jsonArray.stream().map(x -> {
                return ((JSONObject) x).getString("id");
            }).collect(Collectors.toList());
            wrapper = wrapper.in("well_id", id);
        }
        List<Map<String, Object>> maps = new ArrayList<>();
        IPage<Map<String, Object>> pageList = accessWellMapper.selectMapsPage(page,wrapper);
        Map<String, Object> mapPage = new HashMap<>();
        List<Map<String, Object>> list = new ArrayList<>();
        mapPage.put("total",pageList.getTotal());
        mapPage.put("currentPage",pageList.getCurrent());
        mapPage.put("pages",pageList.getPages());
        mapPage.put("pageSize",pageList.getSize());
        List<Map<String, Object>> records = pageList.getRecords();
        for (Map<String, Object> record : records) {
            Map<String, Object> map = new HashMap<>();
            Map hashMap = (Map)cbPcOrganizationService.getOrgInfoByWellID((String) record.get("well_id"));
            if (!Objects.isNull(hashMap)){
                map.put("zyq",hashMap.get("gfatname"));
                map.put("jq",hashMap.get("fatname"));
                map.put("zd",hashMap.get("orgname"));
            }
            map.put("wellId",record.get("well_id"));
            map.put("wellCommonName",record.get("well_common_name"));
            map.put("accessStatus",record.get("access_status").equals(1));
            map.put("accessStatusMsg",record.get("access_status_msg"));
            map.put("remarks",record.get("remarks"));
            map.put("accessDate",record.get("access_date").toString().replace("T"," ").split("\\.")[0]);
            list.add(map);
        }
        mapPage.put("data",list);
        maps.add(mapPage);
        return maps;
    }

    @Override
    public Object addAccessWellInfo(AccessWellInfoVo accessWellInfoVo) {
        if (Objects.isNull(accessWellInfoVo)) return new Object();
        int insert = 0;
        SysAccessWellControl sysAccessWellControl = new SysAccessWellControl();
        if (accessWellInfoVo.getWellId() !=null && !Objects.equals(accessWellInfoVo.getWellId(),"")){
            QueryWrapper<CbCdWellSource> sourceQueryWrapper = new QueryWrapper<CbCdWellSource>().eq("well_id",accessWellInfoVo.getWellId());
            CbCdWellSource cbCdWellSource = cbCdWellSourceMapper.selectOne(sourceQueryWrapper);
            if (!Objects.isNull(cbCdWellSource)){
                sysAccessWellControl.setRemarks(cbCdWellSource.getRemarks());
            }
            if (accessWellInfoVo.getWellCommonName() !=null && !Objects.equals(accessWellInfoVo.getWellCommonName(),"")){
                sysAccessWellControl.setWellId(accessWellInfoVo.getWellId());
                sysAccessWellControl.setWellCommonName(accessWellInfoVo.getWellCommonName());
                sysAccessWellControl.setAccessStatus(accessWellInfoVo.getAccessStatus());
                sysAccessWellControl.setAccessStatusMsg(accessWellInfoVo.getAccessStatus()==0?"未接入":"已接入");
                sysAccessWellControl.setAccessDate(LocalDateTime.now());
                insert = baseMapper.insert(sysAccessWellControl);
            }
        }
        return insert == 1;
    }

    @Override
    public Object updateAccessStatus(AccessWellInfoVo accessWellInfoVo) {
        if (Objects.isNull(accessWellInfoVo)) return new Object();
        SysAccessWellControl sysAccessWellControl = new SysAccessWellControl();
        int update = 0;
        if (accessWellInfoVo.getWellId() !=null && !Objects.equals(accessWellInfoVo.getWellId(),"")){
            if (accessWellInfoVo.getAccessStatus() != null){
                QueryWrapper<SysAccessWellControl> wrapper = new QueryWrapper<SysAccessWellControl>().eq("well_id",accessWellInfoVo.getWellId());
                sysAccessWellControl.setAccessStatus(accessWellInfoVo.getAccessStatus()==0?1:0);
                sysAccessWellControl.setAccessStatusMsg(Objects.equals(accessWellInfoVo.getAccessStatusMsg(), "未接入") ?"已接入":"未接入");
                update = baseMapper.update(sysAccessWellControl, wrapper);
            }
        }
        return update == 1;
    }

    @Override
    public Object updateAccessStatus() {
        QueryWrapper<SysAccessWellControl> wrapper = new QueryWrapper<SysAccessWellControl>().eq("access_status",0).eq("access_status_msg","未接入");
        SysAccessWellControl sysAccessWellControl = new SysAccessWellControl();
        sysAccessWellControl.setAccessStatus(1);
        sysAccessWellControl.setAccessStatusMsg("已接入");
        return baseMapper.update(sysAccessWellControl, wrapper)>0;
    }

    @Override
    public List<SysAccessWellControl> getWellControlList() {
        QueryWrapper<SysAccessWellControl> wrapper = new QueryWrapper<SysAccessWellControl>()
                .eq("error_id",1)
                .eq("access_status",1);
        return baseMapper.selectList(wrapper);
    }

    @Override
    public List<AccessWellExcelVo> exportExcelAccess(SysAccessWellControl sysAccessWellControl) {
        QueryWrapper<SysAccessWellControl> wrapper = new QueryWrapper<SysAccessWellControl>();
        if (!StringUtil.isBlankOrEmpty(sysAccessWellControl.getWellId())){
            wrapper = wrapper.eq("well_id",sysAccessWellControl.getWellId());
        }else {
            if (StringUtil.isBlankOrEmpty(sysAccessWellControl.getOrgId())) sysAccessWellControl.setOrgId("[0,0,3]");
            Object data = cbCdWellSourceService.getjh(sysAccessWellControl.getOrgId()).getData();
            JSONArray jsonArray = (JSONArray) JSONArray.toJSON(data);
            List<Object> id = jsonArray.stream().map(x -> {
                return ((JSONObject) x).getString("id");
            }).collect(Collectors.toList());
            wrapper = wrapper.in("well_id", id);
        }
        List<SysAccessWellControl> list = baseMapper.selectList(wrapper);
        return list.stream().map(x -> {
            AccessWellExcelVo accessWellExcelVo = new AccessWellExcelVo();
            Map hashMap = (Map) cbPcOrganizationService.getOrgInfoByWellID(x.getWellId());
            if (!Objects.isNull(hashMap)) {
                accessWellExcelVo.setZyq(String.valueOf(hashMap.get("gfatname")));
                accessWellExcelVo.setJq(String.valueOf(hashMap.get("fatname")));
                accessWellExcelVo.setZd(String.valueOf(hashMap.get("orgname")));
            }
            accessWellExcelVo.setWellId(x.getWellId());
            accessWellExcelVo.setAccessDate(DateTimeUtil.formatString(x.getAccessDate()));
            String msg = "";
            if (x.getAccessStatus() == 0) msg = "未接入";
            else if (x.getAccessStatus() == 1) msg = "已接入";
            accessWellExcelVo.setAccessStatusMsg(msg);
            accessWellExcelVo.setRemarks(x.getRemarks());
            return accessWellExcelVo;
        }).collect(Collectors.toList());
    }

}
